/** Header file generated by fdesign on Wed Jul  1 11:52:48 2009.**/

#ifndef FD_choiceattrib_h_
#define FD_choiceattrib_h_

#include <forms.h>

/** Callbacks, globals and object handlers **/

extern void add_choice_item_cb( FL_OBJECT *, long );
extern void delete_choice_item_cb( FL_OBJECT *, long );
extern void change_choice_item_cb( FL_OBJECT *, long );
extern void clear_choice_field_cb( FL_OBJECT *, long );
extern void replace_choice_item_cb( FL_OBJECT *, long );
extern void choice_align_cb( FL_OBJECT *, long );
extern void choice_val_cb( FL_OBJECT *, long );
extern void use_struct_cb( FL_OBJECT *, long );
extern void choiceentry_scope_cb( FL_OBJECT *, long );


/**** Forms and Objects ****/

typedef struct {
    FL_FORM   * choiceattrib;
    void      * vdata;
    char      * cdata;
    long        ldata;
    FL_OBJECT * content_br;
    FL_OBJECT * input;
    FL_OBJECT * auto_clear;
    FL_OBJECT * mode;
    FL_OBJECT * shortcut;
    FL_OBJECT * align;
    FL_OBJECT * val;
    FL_OBJECT * new_menuapi;
    FL_OBJECT * scope;
} FD_choiceattrib;

extern FD_choiceattrib * create_form_choiceattrib( void );

#endif /* FD_choiceattrib_h_ */
