/* Form definition file generated by fdesign */

#include <stdlib.h>
#include "spinner_spec.h"


/***************************************
 ***************************************/

FD_spinnerattrib *
create_form_spinnerattrib( void )
{
    FL_OBJECT *obj;
    FD_spinnerattrib *fdui = fl_malloc( sizeof *fdui );

    int old_bw = fl_get_border_width( );
    fl_set_border_width( -1 );

    fdui->vdata = fdui->cdata = NULL;
    fdui->ldata = 0;

    fdui->spinnerattrib = fl_bgn_form( FL_NO_BOX, 540, 280 );

    fdui->background = obj = fl_add_box( FL_FLAT_BOX, 0, 0, 540, 280, "  " );

    obj = fl_add_labelframe( FL_ENGRAVED_FRAME, 25, 20, 485, 235, " Spinner Attributes" );
    fl_set_object_lsize( obj, FL_NORMAL_SIZE );
    fl_set_object_lstyle( obj, FL_BOLD_STYLE );

    fdui->prec = obj = fl_add_counter( FL_SIMPLE_COUNTER, 65, 170, 135, 25, "  Precision" );
    fl_set_object_lsize( obj, FL_NORMAL_SIZE );
    fl_set_object_lalign( obj, FL_ALIGN_TOP );
    fl_set_object_callback( obj, spn_precision_cb, 0 );

    fdui->minval = obj = fl_add_input( FL_FLOAT_INPUT, 340, 60, 92, 24, "Spinner Min." );
    fl_set_object_lsize( obj, FL_NORMAL_SIZE );
    fl_set_object_callback( obj, spn_minmax_change, 0 );
    fl_set_object_return( obj, FL_RETURN_END_CHANGED );

    fdui->maxval = obj = fl_add_input( FL_FLOAT_INPUT, 340, 100, 92, 24, "Spinner Max." );
    fl_set_object_lsize( obj, FL_NORMAL_SIZE );
    fl_set_object_callback( obj, spn_minmax_change, 0 );
    fl_set_object_return( obj, FL_RETURN_END_CHANGED );

    fdui->returnsetting = obj = fl_add_menu( FL_PULLDOWN_MENU, 60, 80, 138, 24, "  Return Setting" );
    fl_set_object_boxtype( obj, FL_UP_BOX );
    fl_set_object_callback( obj, spn_returnsetting_change, 0 );

    fdui->initialval = obj = fl_add_input( FL_FLOAT_INPUT, 340, 150, 92, 24, "Initial Value" );
    fl_set_object_lsize( obj, FL_NORMAL_SIZE );
    fl_set_object_callback( obj, spn_initialvalue_change, 0 );
    fl_set_object_return( obj, FL_RETURN_END_CHANGED );

    fdui->step = obj = fl_add_input( FL_FLOAT_INPUT, 340, 200, 92, 24, "Step size" );
    fl_set_object_lsize( obj, FL_NORMAL_SIZE );
    fl_set_object_callback( obj, spn_stepchange_cb, 0 );
    fl_set_object_return( obj, FL_RETURN_END_CHANGED );

    fl_end_form( );

    fdui->spinnerattrib->fdui = fdui;
    fl_set_border_width( old_bw );

    return fdui;
}
