/* Form definition file generated by fdesign. */

#include <stdlib.h>
#include "ui_objs.h"


/***************************************
 ***************************************/

FD_buttonform *
create_form_buttonform( void )
{
    FL_OBJECT *obj;
    FD_buttonform *fdui = fl_malloc( sizeof *fdui );

    int old_bw = fl_get_border_width( );
    fl_set_border_width( -1 );

    fdui->vdata = fdui->cdata = NULL;
    fdui->ldata = 0;

    fdui->buttonform = fl_bgn_form( FL_NO_BOX, 581, 51 );

    obj = fl_add_box( FL_FLAT_BOX, 0, 0, 581, 51, "" );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 20, 3, 30, 30, "button" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 60, 3, 30, 30, "round" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 100, 3, 30, 30, "round3d" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 140, 3, 30, 30, "check" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 180, 3, 30, 30, "light" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 220, 3, 30, 30, "bitmap" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 260, 3, 30, 30, "pixmap" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 300, 3, 30, 30, "scroll" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    fl_end_form( );

    fdui->buttonform->fdui = fdui;
    fl_set_border_width( old_bw );

    return fdui;
}


/***************************************
 ***************************************/

FD_valuatorform *
create_form_valuatorform( void )
{
    FL_OBJECT *obj;
    FD_valuatorform *fdui = fl_malloc( sizeof *fdui );

    int old_bw = fl_get_border_width( );
    fl_set_border_width( -1 );

    fdui->vdata = fdui->cdata = NULL;
    fdui->ldata = 0;

    fdui->valuatorform = fl_bgn_form( FL_NO_BOX, 581, 51 );

    obj = fl_add_box( FL_FLAT_BOX, 0, 0, 581, 51, "" );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 20, 5, 30, 30, "slider" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 66, 5, 30, 30, "valslider" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 112, 5, 30, 30, "scrollbar" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 158, 5, 30, 30, "dial" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 204, 5, 30, 30, "counter" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 250, 5, 30, 30, "positioner" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    fl_end_form( );

    fdui->valuatorform->fdui = fdui;
    fl_set_border_width( old_bw );

    return fdui;
}


/***************************************
 ***************************************/

FD_staticform *
create_form_staticform( void )
{
    FL_OBJECT *obj;
    FD_staticform *fdui = fl_malloc( sizeof *fdui );

    int old_bw = fl_get_border_width( );
    fl_set_border_width( -1 );

    fdui->vdata = fdui->cdata = NULL;
    fdui->ldata = 0;

    fdui->staticform = fl_bgn_form( FL_NO_BOX, 581, 51 );

    obj = fl_add_box( FL_FLAT_BOX, 0, 0, 581, 51, "" );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 20, 3, 30, 30, "box" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 70, 3, 30, 30, "frame" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 120, 3, 30, 30, "labelframe" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 170, 3, 30, 30, "text" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 220, 3, 30, 30, "bitmap" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 270, 3, 30, 30, "pixmap" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 320, 3, 30, 30, "chart" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    obj = fl_add_pixmapbutton( FL_NORMAL_BUTTON, 370, 3, 30, 30, "clock" );
    fl_set_object_color( obj, FL_COL1, FL_YELLOW );

    fl_end_form( );

    fdui->staticform->fdui = fdui;
    fl_set_border_width( old_bw );

    return fdui;
}
