/** Header file generated by fdesign on Wed Jul  1 11:52:44 2009.**/

#ifndef FD_menuattrib_h_
#define FD_menuattrib_h_

#include <forms.h>

/** Callbacks, globals and object handlers **/

extern void add_menu_item_cb( FL_OBJECT *, long );
extern void delete_menu_item_cb( FL_OBJECT *, long );
extern void change_menu_item_cb( FL_OBJECT *, long );
extern void clear_menu_field_cb( FL_OBJECT *, long );
extern void replace_menu_item_cb( FL_OBJECT *, long );
extern void new_menuapi_cb( FL_OBJECT *, long );
extern void menuentry_scope_cb( FL_OBJECT *, long );


/**** Forms and Objects ****/

typedef struct {
    FL_FORM   * menuattrib;
    void      * vdata;
    char      * cdata;
    long        ldata;
    FL_OBJECT * content_br;
    FL_OBJECT * input;
    FL_OBJECT * auto_clear;
    FL_OBJECT * mode;
    FL_OBJECT * shortcut;
    FL_OBJECT * item_cb;
    FL_OBJECT * new_menuapi;
    FL_OBJECT * scope;
    FL_OBJECT * id;
} FD_menuattrib;

extern FD_menuattrib * create_form_menuattrib( void );

#endif /* FD_menuattrib_h_ */
