/** Header file generated by fdesign on Tue Jun  9 17:10:25 2009.**/

#ifndef FD_buttonattrib_h_
#define FD_buttonattrib_h_

#include <forms.h>

/** Callbacks, globals and object handlers **/

extern void initialval_change( FL_OBJECT *, long );
extern void iconbutton_filename_change( FL_OBJECT *, long );
extern void pixalign_change( FL_OBJECT *, long );
extern void showfocus_change( FL_OBJECT *, long );
extern void lookfor_pixmapfile_cb( FL_OBJECT *, long );
extern void usedata_change( FL_OBJECT *, long );
extern void fullpath_cb( FL_OBJECT *, long );


/**** Forms and Objects ****/

typedef struct {
    FL_FORM   * buttonattrib;
    void      * vdata;
    char      * cdata;
    long        ldata;
    FL_OBJECT * background;
    FL_OBJECT * initialval;
    FL_OBJECT * filename;
    FL_OBJECT * pixalign;
    FL_OBJECT * showfocus;
    FL_OBJECT * browse;
    FL_OBJECT * use_data;
    FL_OBJECT * fullpath;
} FD_buttonattrib;

extern FD_buttonattrib * create_form_buttonattrib( void );

#endif /* FD_buttonattrib_h_ */
