/** Header file generated by fdesign on Tue Jun  9 17:10:25 2009.**/

#ifndef FD_pixmapattrib_h_
#define FD_pixmapattrib_h_

#include <forms.h>

/** Callbacks, globals and object handlers **/

extern void pixmap_filename_change( FL_OBJECT *, long );
extern void pixmapalign_change( FL_OBJECT *, long );
extern void pixmaplookfor_pixmapfile_cb( FL_OBJECT *, long );
extern void pixmapusedata_change( FL_OBJECT *, long );
extern void pixmapfullpath_cb( FL_OBJECT *, long );


/**** Forms and Objects ****/

typedef struct {
    FL_FORM   * pixmapattrib;
    void      * vdata;
    char      * cdata;
    long        ldata;
    FL_OBJECT * background;
    FL_OBJECT * filename;
    FL_OBJECT * pixalign;
    FL_OBJECT * browse;
    FL_OBJECT * use_data;
    FL_OBJECT * fullpath;
} FD_pixmapattrib;

extern FD_pixmapattrib * create_form_pixmapattrib( void );

#endif /* FD_pixmapattrib_h_ */
